/*
 * Application processor startup code, moved from smp.c to better support kernel profile
 *
 * Copyright (C) 1999 Walt Drummond <drummond@valinux.com>
 * Copyright (C) 1999, 2001 David Mosberger-Tang <davidm@hpl.hp.com>
 * Copyright (C) 2000 Asit Mallick <Asit.K.Mallick@intel.com>
 */

#include <asm/kregs.h>
#include <asm/pgtable.h>
#include <asm/processor.h>

#define	MB (1024*1024)

/* 
 * SAL shoves the AP's here when we start them.  Physical mode, no kernel TR, 
 * no RRs set, better than even chance that psr is bogus.  Fix all that and 
 * call _start.  In effect, pretend to be lilo.
 *
 * Stolen from lilo_start.c.  Thanks David! 
 */
void
start_ap (void)
{
	extern void _start (void);
	unsigned long flags;

	/*
	 * Install translation registers that identity map:
	 *	kernel's 64MB code page.
	 *	kernel's 256MB data page.
	 */
	ia64_clear_ic(flags);
	ia64_set_rr(PAGE_OFFSET, (ia64_rid(0, PAGE_OFFSET) << 8) | (_PAGE_SIZE_256M << 2));
	ia64_srlz_d();
	// code
	ia64_itr(0x1, IA64_TR_KERNEL, PAGE_OFFSET+64*MB,
		 pte_val(mk_pte_phys(64*MB, __pgprot(__DIRTY_BITS|_PAGE_PL_0|_PAGE_AR_RWX))),
		 _PAGE_SIZE_64M);
	// data
	ia64_itr(0x2, IA64_TR_KERNEL, PAGE_OFFSET,
		 pte_val(mk_pte_phys(0, __pgprot(__DIRTY_BITS|_PAGE_PL_0|_PAGE_AR_RWX))),
		 _PAGE_SIZE_256M);
	ia64_srlz_i();

	flags = (IA64_PSR_IT | IA64_PSR_IC | IA64_PSR_DT | IA64_PSR_RT | IA64_PSR_DFH | 
		 IA64_PSR_BN);
	
	asm volatile ("movl r8 = 1f\n"
		      ";;\n"
		      "mov cr.ipsr=%0\n"
		      "mov cr.iip=r8\n" 
		      "mov cr.ifs=r0\n"
		      ";;\n"
		      "rfi;;"
		      "1:\n"
		      "movl r1 = __gp" :: "r"(flags) : "r8");
	_start();
}


